#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <errno.h>

#define LINUX

/** shared memory utilities **/
#define MAXSHM 20		/* maximum number of memory chunks */
#define MAXMEM 1024		/* maximum amount of memory to be used 
				 * note: there is a system limit, too*/

static int cShm = 0;		/* count of shared memory regions */
static int cByteOffset = 0;	/* byte offset from sbShm */
static char *sbShmBase = NULL;	/* base location of shared memory chunk */
static int shm = -1;		/* shm value for shared memory chunk */

static char *rgsbShm[MAXSHM];	/* keep track of memory pointers */

/** semaphore utilities **/
#if defined(ns32000) || defined(mips) || defined(LINUX)
union semun {
    int val;
    struct semid_ds *buf;
    u_short *array;
};
#endif

extern int errno;

#define MAXSEM 15		/* maximum number of semaphores */

static int semBase = -1;	/* base semaphore value */
static int cSem = 0;		/* count of current semaphores */
static int iSem = 0;		/* current index of semaphore to use */
static int rgSem[MAXSEM];	/* keep track of semaphores used */

/** function prototypes **/

/*
 * shmcreate() - create the given amount of shared memory and return the 
 *	pointer to this memory.  Return NULL on failure.  We have imposed
 *	a limit of 1024 bytes for the total amount of shared memory created.
 *      Attaches if the key exists.
 */
char *shmcreate(key_t key, int cnt /* number of bytes */);

/*
 * shmdelete() - delete the shared memory associated with the given address.
 *	Return -1 on failure, zero on success.
 */
int shmdelete(char *sbShm);

/*
 * semwait() - wait on the given semaphore.  Return -1 on error, zero on
 *	success.
 */
int semwait(int sem);

/*
 * semwaitn() - wait on the given semaphore for the given count.  Return -1 on 
 *	error, zero on success.
 */
int semwaitn(int sem, int cnt);

/*
 * semsignal() - signal the given semaphore.  Returns -1 on failure.
 */
int semsignal(int sem);

/*
 * semsignaln() - signal the given semaphore, the given number of times.  Returns
 *	-1 on failure.
 */
int semsignaln(int sem, int cnt);

/*
 * screate() - create a semaphore with the given initial count and the key.  
 *      Returns -1 on failure, a semaphore id on success.  
 *      We have imposed a limit 12 semaphores that can be created.
 */
int screate(key_t key, int cnt);

/*
 * sdelete() - delete the semaphore corresponding to the given sem id.  Returns
 *	-1 on failure, zero on success. 
 */
int sdelete(int sem);

/*
 * scount() - return the count of the given sem id.  -1 on failure.
 */
int scount(int sem);

/*
 * sreset() - reset the count of the given sem id.  Return -1 on failure,
 *	zero on success.
 */
int sreset(int sem, int cnt);

/*
 * sgetval() - get the semaphore integer value.
 */
int sgetval(int sem);
